@@ -230,6 +230,8 @@ type (
 
 	// DataStore is the configuration for a single datastore
 	DataStore struct {
+		// FaultInjection contains the config for fault injector wrapper.
+		FaultInjection *FaultInjection `yaml:"faultInjection"`
 		// Cassandra contains the config for a cassandra datastore
 		Cassandra *Cassandra `yaml:"cassandra"`
 		// SQL contains the config for a SQL based datastore
@@ -240,6 +242,10 @@ type (
 		ElasticSearch *Elasticsearch `yaml:"elasticsearch"`
 	}
 
+	FaultInjection struct {
+		Rate float64 `yaml:"rate"`
+	}
+
 	// VisibilityConfig is config for visibility sampling
 	VisibilityConfig struct {
 		// EnableSampling for visibility
@@ -250,8 +256,6 @@ type (
 		VisibilityClosedMaxQPS dynamicconfig.IntPropertyFnWithNamespaceFilter `yaml:"-" json:"-"`
 		// VisibilityListMaxQPS max QPS for list workflow
 		VisibilityListMaxQPS dynamicconfig.IntPropertyFnWithNamespaceFilter `yaml:"-" json:"-"`
-		// ESIndexMaxResultWindow ElasticSearch index setting max_result_window
-		ESIndexMaxResultWindow dynamicconfig.IntPropertyFn `yaml:"-" json:"-"`
 		// MaxQPS is overall max QPS
 		MaxQPS dynamicconfig.IntPropertyFn `yaml:"-" json:"-"`
 		// ESProcessorAckTimeout is the timeout that store will wait to get ack signal from ES processor.
