@@ -610,7 +610,7 @@ func createTransferTasks(
 			ScheduleId:              scheduleID,
 			Version:                 task.GetVersion(),
 			TaskId:                  task.GetTaskID(),
-			VisibilityTime:          timestamp.TimePtr(task.GetVisibilityTimestamp()),
+			VisibilityTime:          timestamp.TimePtr(task.GetVisibilityTime()),
 		}
 
 		dataBlob, err := serialization.TransferTaskInfoToBlob(transferTaskInfo)
@@ -716,7 +716,7 @@ func createVisibilityTasks(
 			TaskId:         task.GetTaskID(),
 			TaskType:       task.GetType(),
 			Version:        task.GetVersion(),
-			VisibilityTime: timestamp.TimePtr(task.GetVisibilityTimestamp()),
+			VisibilityTime: timestamp.TimePtr(task.GetVisibilityTime()),
 		})
 		if err != nil {
 			return err
@@ -802,7 +802,7 @@ func createTimerTasks(
 		}
 
 		// Ignoring possible type cast errors.
-		goTs := task.GetVisibilityTimestamp()
+		goTs := task.GetVisibilityTime()
 		dbTs := p.UnixMilliseconds(goTs)
 
 		datablob, err := serialization.TimerTaskInfoToBlob(&persistencespb.TimerTaskInfo{
@@ -837,76 +837,6 @@ func createTimerTasks(
 	return nil
 }
 
-func createOrUpdateCurrentExecution(
-	batch gocql.Batch,
-	createMode p.CreateWorkflowMode,
-	shardID int32,
-	namespaceID string,
-	workflowID string,
-	runID string,
-	state enumsspb.WorkflowExecutionState,
-	lastWriteVersion int64,
-	previousRunID string,
-	previousLastWriteVersion int64,
-	executionStateBlob *commonpb.DataBlob,
-) error {
-	switch createMode {
-	case p.CreateWorkflowModeContinueAsNew:
-		batch.Query(templateUpdateCurrentWorkflowExecutionQuery,
-			runID,
-			executionStateBlob.Data,
-			executionStateBlob.EncodingType.String(),
-			lastWriteVersion,
-			state,
-			shardID,
-			rowTypeExecution,
-			namespaceID,
-			workflowID,
-			permanentRunID,
-			defaultVisibilityTimestamp,
-			rowTypeExecutionTaskID,
-			previousRunID,
-		)
-	case p.CreateWorkflowModeWorkflowIDReuse:
-		batch.Query(templateUpdateCurrentWorkflowExecutionForNewQuery,
-			runID,
-			executionStateBlob.Data,
-			executionStateBlob.EncodingType.String(),
-			lastWriteVersion,
-			state,
-			shardID,
-			rowTypeExecution,
-			namespaceID,
-			workflowID,
-			permanentRunID,
-			defaultVisibilityTimestamp,
-			rowTypeExecutionTaskID,
-			previousRunID,
-			previousLastWriteVersion,
-			enumsspb.WORKFLOW_EXECUTION_STATE_COMPLETED,
-		)
-	case p.CreateWorkflowModeBrandNew:
-		batch.Query(templateCreateCurrentWorkflowExecutionQuery,
-			shardID,
-			rowTypeExecution,
-			namespaceID,
-			workflowID,
-			permanentRunID,
-			defaultVisibilityTimestamp,
-			rowTypeExecutionTaskID,
-			runID,
-			executionStateBlob.Data,
-			executionStateBlob.EncodingType.String(),
-			lastWriteVersion,
-			state,
-		)
-	default:
-		return serviceerror.NewInternal(fmt.Sprintf("unknown mode: %v", createMode))
-	}
-
-	return nil
-}
-
 func updateActivityInfos(
 	batch gocql.Batch,
 	activityInfos map[int64]*commonpb.DataBlob,
