@@ -58,19 +58,19 @@ type (
 		suite.Suite
 		*require.Assertions
 
-		store  p.HistoryManager
+		store  p.ExecutionManager
 		logger log.Logger
 	}
 )
 
 func newHistoryEventsSuite(
 	t *testing.T,
-	store p.HistoryStore,
+	store p.ExecutionStore,
 	logger log.Logger,
 ) *historyEventsSuite {
 	return &historyEventsSuite{
 		Assertions: require.New(t),
-		store: p.NewHistoryV2ManagerImpl(
+		store: p.NewExecutionManager(
 			store,
 			logger,
 			dynamicconfig.GetIntPropertyFn(4*1024*1024),
