@@ -77,8 +77,9 @@ type (
 
 	ackChan struct { // value of processorImpl.mapToAckChan
 		ackChInternal chan bool
-		addedAt       time.Time // Time when request was added to bulk processor (used to report metrics).
-		startedAt     time.Time // Time when request was sent to Elasticsearch by bulk processor (used to report metrics).
+		createdAt     time.Time    // Time when request was created (used to report metrics).
+		addedAt       atomic.Value // of time.Time // Time when request was added to bulk processor (used to report metrics).
+		startedAt     time.Time    // Time when request was sent to Elasticsearch by bulk processor (used to report metrics).
 	}
 )
 
@@ -166,29 +167,24 @@ func (p *processorImpl) hashFn(key interface{}) uint32 {
 // Add request to the bulk and return ack channel which will receive ack signal when request is processed.
 func (p *processorImpl) Add(request *esclient.BulkableRequest, visibilityTaskKey string) <-chan bool {
 	ackCh := newAckChan()
-	retCh := ackCh.ackChInternal
 	_, isDup, _ := p.mapToAckChan.PutOrDo(visibilityTaskKey, ackCh, func(key interface{}, value interface{}) error {
 		ackChExisting, ok := value.(*ackChan)
 		if !ok {
 			p.logger.Fatal(fmt.Sprintf("mapToAckChan has item of a wrong type %T (%T expected).", value, &ackChan{}), tag.Value(key))
 		}
 
-		p.logger.Warn("Adding duplicate ES request for visibility task key.", tag.Key(visibilityTaskKey), tag.ESDocID(request.ID), tag.Value(request.Doc))
+		p.logger.Warn("Skipping duplicate ES request for visibility task key.", tag.Key(visibilityTaskKey), tag.ESDocID(request.ID), tag.Value(request.Doc), tag.NewDurationTag("interval-between-duplicates", ackCh.createdAt.Sub(ackChExisting.createdAt)))
+		p.metricsClient.IncCounter(metrics.ElasticsearchBulkProcessor, metrics.ElasticsearchBulkProcessorDuplicateRequest)
 
-		// Ack existing visibility task as if it is processed successfully.
-		ackChExisting.done(true, p.metricsClient)
-
-		// Replace existing dictionary item with new item.
-		// Note: request won't be added to bulk processor.
-		ackChExisting.addedAt = ackCh.addedAt
-		ackChExisting.startedAt = ackCh.startedAt
-		ackChExisting.ackChInternal = ackCh.ackChInternal
+		// Ack duplicate visibility task right away as if it is processed successfully.
+		ackCh.done(true, p.metricsClient)
 		return nil
 	})
 	if !isDup {
 		p.bulkProcessor.Add(request)
+		ackCh.recordAdd(p.metricsClient)
 	}
-	return retCh
+	return ackCh.ackChInternal
 }
 
 // bulkBeforeAction is triggered before bulk processor commit
@@ -207,7 +203,7 @@ func (p *processorImpl) bulkBeforeAction(_ int64, requests []elastic.BulkableReq
 			if !ok {
 				p.logger.Fatal(fmt.Sprintf("mapToAckChan has item of a wrong type %T (%T expected).", value, &ackChan{}), tag.Value(key))
 			}
-			ackCh.start(p.metricsClient)
+			ackCh.recordStart(p.metricsClient)
 			return nil
 		})
 	}
@@ -395,23 +391,38 @@ func extractErrorReason(resp *elastic.BulkResponseItem) string {
 }
 
 func newAckChan() *ackChan {
+	var addedAt atomic.Value
+	addedAt.Store(time.Time{})
 	return &ackChan{
 		ackChInternal: make(chan bool, 1),
-		addedAt:       time.Now().UTC(),
+		createdAt:     time.Now().UTC(),
+		addedAt:       addedAt,
 	}
 }
 
-func (a *ackChan) start(metricsClient metrics.Client) {
-	metricsClient.RecordTimer(metrics.ElasticsearchBulkProcessor, metrics.ElasticsearchBulkProcessorWaitLatency, time.Now().UTC().Sub(a.addedAt))
+func (a *ackChan) recordAdd(metricsClient metrics.Client) {
+	addedAt := time.Now().UTC()
+	a.addedAt.Store(addedAt)
+	metricsClient.RecordTimer(metrics.ElasticsearchBulkProcessor, metrics.ElasticsearchBulkProcessorWaitAddLatency, addedAt.Sub(a.createdAt))
+}
+
+func (a *ackChan) recordStart(metricsClient metrics.Client) {
 	a.startedAt = time.Now().UTC()
+	addedAt := a.addedAt.Load().(time.Time)
+	if !addedAt.IsZero() {
+		metricsClient.RecordTimer(metrics.ElasticsearchBulkProcessor, metrics.ElasticsearchBulkProcessorWaitStartLatency, a.startedAt.Sub(addedAt))
+	}
 }
 
 func (a *ackChan) done(ack bool, metricsClient metrics.Client) {
 	select {
 	case a.ackChInternal <- ack:
-		metricsClient.RecordTimer(metrics.ElasticsearchBulkProcessor, metrics.ElasticsearchBulkProcessorRequestLatency, time.Now().UTC().Sub(a.addedAt))
+		doneAt := time.Now().UTC()
+		if !a.createdAt.IsZero() {
+			metricsClient.RecordTimer(metrics.ElasticsearchBulkProcessor, metrics.ElasticsearchBulkProcessorRequestLatency, doneAt.Sub(a.createdAt))
+		}
 		if !a.startedAt.IsZero() {
-			metricsClient.RecordTimer(metrics.ElasticsearchBulkProcessor, metrics.ElasticsearchBulkProcessorCommitLatency, time.Now().UTC().Sub(a.startedAt))
+			metricsClient.RecordTimer(metrics.ElasticsearchBulkProcessor, metrics.ElasticsearchBulkProcessorCommitLatency, doneAt.Sub(a.startedAt))
 		}
 	default:
 		metricsClient.IncCounter(metrics.ElasticsearchBulkProcessor, metrics.ElasticsearchBulkProcessorDeadlock)
