@@ -86,6 +86,7 @@ type (
 		clusterMetadata cluster.Metadata
 		saProvider      searchattribute.Provider
 		saManager       searchattribute.Manager
+		saMapper        searchattribute.Mapper
 
 		// other common resources
 
@@ -247,7 +248,6 @@ func New(
 	}
 
 	saProvider := persistence.NewSearchAttributesManager(clock.NewRealTimeSource(), persistenceBean.GetClusterMetadataManager())
-
 	saManager := persistence.NewSearchAttributesManager(clock.NewRealTimeSource(), persistenceBean.GetClusterMetadataManager())
 
 	visibilityMgr, err := visibilityManagerInitializer(
@@ -323,6 +323,7 @@ func New(
 		clusterMetadata: clusterMetadata,
 		saProvider:      saProvider,
 		saManager:       saManager,
+		saMapper:        params.SearchAttributesMapper,
 
 		// other common resources
 
@@ -633,3 +634,7 @@ func (h *Impl) GetSearchAttributesProvider() searchattribute.Provider {
 func (h *Impl) GetSearchAttributesManager() searchattribute.Manager {
 	return h.saManager
 }
+
+func (h *Impl) GetSearchAttributesMapper() searchattribute.Mapper {
+	return h.saMapper
+}
