@@ -29,20 +29,18 @@ import (
 	"fmt"
 
 	commonpb "go.temporal.io/api/common/v1"
+	enumspb "go.temporal.io/api/enums/v1"
 	"go.temporal.io/api/serviceerror"
 
 	"go.temporal.io/server/common/dynamicconfig"
-	"go.temporal.io/server/common/log"
-	"go.temporal.io/server/common/log/tag"
 	"go.temporal.io/server/common/payload"
 )
 
 type (
 	// Validator is used to validate search attributes
 	Validator struct {
-		logger log.Logger
-
 		searchAttributesProvider          Provider
+		searchAttributesMapper            Mapper
 		searchAttributesNumberOfKeysLimit dynamicconfig.IntPropertyFnWithNamespaceFilter
 		searchAttributesSizeOfValueLimit  dynamicconfig.IntPropertyFnWithNamespaceFilter
 		searchAttributesTotalSizeLimit    dynamicconfig.IntPropertyFnWithNamespaceFilter
@@ -55,31 +53,22 @@ var (
 
 // NewValidator create Validator
 func NewValidator(
-	logger log.Logger,
 	searchAttributesProvider Provider,
+	searchAttributesMapper Mapper,
 	searchAttributesNumberOfKeysLimit dynamicconfig.IntPropertyFnWithNamespaceFilter,
 	searchAttributesSizeOfValueLimit dynamicconfig.IntPropertyFnWithNamespaceFilter,
 	searchAttributesTotalSizeLimit dynamicconfig.IntPropertyFnWithNamespaceFilter,
 ) *Validator {
 	return &Validator{
-		logger:                            logger,
 		searchAttributesProvider:          searchAttributesProvider,
+		searchAttributesMapper:            searchAttributesMapper,
 		searchAttributesNumberOfKeysLimit: searchAttributesNumberOfKeysLimit,
 		searchAttributesSizeOfValueLimit:  searchAttributesSizeOfValueLimit,
 		searchAttributesTotalSizeLimit:    searchAttributesTotalSizeLimit,
 	}
 }
 
-// ValidateAndLog validate search attributes are valid for writing and not exceed limits
-func (v *Validator) ValidateAndLog(searchAttributes *commonpb.SearchAttributes, namespace string, indexName string) error {
-	err := v.Validate(searchAttributes, namespace, indexName)
-	if err != nil {
-		v.logger.Warn("Search attributes are invalid.", tag.Error(err), tag.WorkflowNamespace(namespace), tag.ESIndex(indexName))
-	}
-	return err
-}
-
-// Validate validate search attributes are valid for writing.
+// Validate search attributes are valid for writing.
 func (v *Validator) Validate(searchAttributes *commonpb.SearchAttributes, namespace string, indexName string) error {
 	if searchAttributes == nil {
 		return nil
@@ -90,28 +79,35 @@ func (v *Validator) Validate(searchAttributes *commonpb.SearchAttributes, namesp
 		return serviceerror.NewInvalidArgument(fmt.Sprintf("number of search attributes %d exceeds limit %d", lengthOfFields, v.searchAttributesNumberOfKeysLimit(namespace)))
 	}
 
-	typeMap, err := v.searchAttributesProvider.GetSearchAttributes(indexName, false)
+	saTypeMap, err := v.searchAttributesProvider.GetSearchAttributes(indexName, false)
 	if err != nil {
 		return serviceerror.NewInvalidArgument(fmt.Sprintf("unable to get search attributes from cluster metadata: %v", err))
 	}
 
 	for saName, saPayload := range searchAttributes.GetIndexedFields() {
-		if _, err := typeMap.getType(saName, systemCategory); err == nil {
+		if _, err = saTypeMap.getType(saName, systemCategory); err == nil {
 			return serviceerror.NewInvalidArgument(fmt.Sprintf("%s attribute can't be set in SearchAttributes", saName))
 		}
 
-		saType, err := typeMap.getType(saName, customCategory|predefinedCategory)
-		if err != nil {
+		fieldName := saName
+		if IsMappable(saName) && v.searchAttributesMapper != nil {
+			fieldName, err = v.searchAttributesMapper.GetFieldName(saName, namespace)
+			if err != nil {
+				return err
+			}
+		}
+
+		var saType enumspb.IndexedValueType
+		if saType, err = saTypeMap.getType(fieldName, customCategory|predefinedCategory); err != nil {
 			if errors.Is(err, ErrInvalidName) {
 				return serviceerror.NewInvalidArgument(fmt.Sprintf("%s is not a valid search attribute name", saName))
 			}
 			return serviceerror.NewInvalidArgument(fmt.Sprintf("unable to get %s search attribute type: %v", saName, err))
 		}
 
-		_, err = DecodeValue(saPayload, saType)
-		if err != nil {
+		if _, err = DecodeValue(saPayload, saType); err != nil {
 			var invalidValue interface{}
-			if err := payload.Decode(saPayload, &invalidValue); err != nil {
+			if err = payload.Decode(saPayload, &invalidValue); err != nil {
 				invalidValue = fmt.Sprintf("value from <%s>", saPayload.String())
 			}
 			return serviceerror.NewInvalidArgument(fmt.Sprintf("%v is not a valid value for search attribute %s of type %s", invalidValue, saName, saType))
