@@ -43,7 +43,6 @@ import (
 	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/cluster"
 	"go.temporal.io/server/common/dynamicconfig"
-	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/payloads"
 	"go.temporal.io/server/common/primitives/timestamp"
 	"go.temporal.io/server/common/searchattribute"
@@ -97,8 +96,8 @@ func (s *commandAttrValidatorSuite) SetupTest() {
 		s.mockNamespaceCache,
 		config,
 		searchattribute.NewValidator(
-			log.NewNoopLogger(),
 			searchattribute.NewTestProvider(),
+			nil,
 			config.SearchAttributesNumberOfKeysLimit,
 			config.SearchAttributesSizeOfValueLimit,
 			config.SearchAttributesTotalSizeLimit,
