@@ -33,7 +33,6 @@ import (
 
 	"go.temporal.io/api/serviceerror"
 
-	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/persistence"
 	"go.temporal.io/server/service/history/workflow"
 )
@@ -167,14 +166,6 @@ func (r *nDCTransactionMgrForNewWorkflowImpl) createAsCurrent(
 		return serviceerror.NewInternal("unable to create 1st event batch")
 	}
 
-	targetWorkflowHistorySize, err := r.persistNewNDCWorkflowEvents(
-		targetWorkflow,
-		targetWorkflowEventsSeq[0],
-	)
-	if err != nil {
-		return err
-	}
-
 	// target workflow to be created as current
 	if currentWorkflow != nil {
 		// current workflow exists, need to do compare and swap
@@ -191,7 +182,7 @@ func (r *nDCTransactionMgrForNewWorkflowImpl) createAsCurrent(
 			prevLastWriteVersion,
 			targetWorkflow.getMutableState(),
 			targetWorkflowSnapshot,
-			targetWorkflowHistorySize,
+			targetWorkflowEventsSeq,
 		)
 	}
 
@@ -206,7 +197,7 @@ func (r *nDCTransactionMgrForNewWorkflowImpl) createAsCurrent(
 		prevLastWriteVersion,
 		targetWorkflow.getMutableState(),
 		targetWorkflowSnapshot,
-		targetWorkflowHistorySize,
+		targetWorkflowEventsSeq,
 	)
 }
 
@@ -243,14 +234,6 @@ func (r *nDCTransactionMgrForNewWorkflowImpl) createAsZombie(
 		return serviceerror.NewInternal("unable to create 1st event batch")
 	}
 
-	targetWorkflowHistorySize, err := r.persistNewNDCWorkflowEvents(
-		targetWorkflow,
-		targetWorkflowEventsSeq[0],
-	)
-	if err != nil {
-		return err
-	}
-
 	if err := targetWorkflow.getContext().ReapplyEvents(
 		targetWorkflowEventsSeq,
 	); err != nil {
@@ -267,12 +250,12 @@ func (r *nDCTransactionMgrForNewWorkflowImpl) createAsZombie(
 		prevLastWriteVersion,
 		targetWorkflow.getMutableState(),
 		targetWorkflowSnapshot,
-		targetWorkflowHistorySize,
+		targetWorkflowEventsSeq,
 	)
 	switch err.(type) {
 	case nil:
 		return nil
-	case *persistence.WorkflowExecutionAlreadyStartedError:
+	case *persistence.WorkflowConditionFailedError:
 		// workflow already created
 		return nil
 	default:
@@ -372,10 +355,5 @@ func (r *nDCTransactionMgrForNewWorkflowImpl) persistNewNDCWorkflowEvents(
 	targetNewWorkflow nDCWorkflow,
 	targetNewWorkflowEvents *persistence.WorkflowEvents,
 ) (int64, error) {
-
-	firstEventID := targetNewWorkflowEvents.Events[0].EventId
-	if firstEventID == common.FirstEventID {
-		return targetNewWorkflow.getContext().PersistFirstWorkflowEvents(targetNewWorkflowEvents)
-	}
-	return targetNewWorkflow.getContext().PersistNonFirstWorkflowEvents(targetNewWorkflowEvents)
+	return targetNewWorkflow.getContext().PersistWorkflowEvents(targetNewWorkflowEvents)
 }
