@@ -111,7 +111,7 @@ func (s *queueTaskSuite) TestExecute_ExecutionErr() {
 	})
 
 	executionErr := errors.New("some random error")
-	s.mockQueueTaskExecutor.EXPECT().execute(queueTaskBase.queueTaskInfo, true).Return(executionErr)
+	s.mockQueueTaskExecutor.EXPECT().execute(gomock.Any(), queueTaskBase.queueTaskInfo, true).Return(executionErr)
 
 	err := queueTaskBase.Execute()
 	s.Equal(executionErr, err)
@@ -122,7 +122,7 @@ func (s *queueTaskSuite) TestExecute_Success() {
 		return true, nil
 	})
 
-	s.mockQueueTaskExecutor.EXPECT().execute(queueTaskBase.queueTaskInfo, true).Return(nil)
+	s.mockQueueTaskExecutor.EXPECT().execute(gomock.Any(), queueTaskBase.queueTaskInfo, true).Return(nil)
 
 	err := queueTaskBase.Execute()
 	s.NoError(err)
