@@ -81,6 +81,7 @@ func newTransferQueueStandbyTaskExecutor(
 }
 
 func (t *transferQueueStandbyTaskExecutor) execute(
+	ctx context.Context,
 	taskInfo queueTaskInfo,
 	shouldProcessTask bool,
 ) error {
@@ -98,17 +99,17 @@ func (t *transferQueueStandbyTaskExecutor) execute(
 
 	switch transferTask.TaskType {
 	case enumsspb.TASK_TYPE_TRANSFER_ACTIVITY_TASK:
-		return t.processActivityTask(transferTask)
+		return t.processActivityTask(ctx, transferTask)
 	case enumsspb.TASK_TYPE_TRANSFER_WORKFLOW_TASK:
-		return t.processWorkflowTask(transferTask)
+		return t.processWorkflowTask(ctx, transferTask)
 	case enumsspb.TASK_TYPE_TRANSFER_CLOSE_EXECUTION:
-		return t.processCloseExecution(transferTask)
+		return t.processCloseExecution(ctx, transferTask)
 	case enumsspb.TASK_TYPE_TRANSFER_CANCEL_EXECUTION:
-		return t.processCancelExecution(transferTask)
+		return t.processCancelExecution(ctx, transferTask)
 	case enumsspb.TASK_TYPE_TRANSFER_SIGNAL_EXECUTION:
-		return t.processSignalExecution(transferTask)
+		return t.processSignalExecution(ctx, transferTask)
 	case enumsspb.TASK_TYPE_TRANSFER_START_CHILD_EXECUTION:
-		return t.processStartChildExecution(transferTask)
+		return t.processStartChildExecution(ctx, transferTask)
 	case enumsspb.TASK_TYPE_TRANSFER_RESET_WORKFLOW:
 		// no reset needed for standby
 		// TODO: add error logs
@@ -119,6 +120,7 @@ func (t *transferQueueStandbyTaskExecutor) execute(
 }
 
 func (t *transferQueueStandbyTaskExecutor) processActivityTask(
+	ctx context.Context,
 	transferTask *persistencespb.TransferTaskInfo,
 ) error {
 
@@ -143,6 +145,7 @@ func (t *transferQueueStandbyTaskExecutor) processActivityTask(
 	}
 
 	return t.processTransfer(
+		ctx,
 		processTaskIfClosed,
 		transferTask,
 		actionFn,
@@ -158,6 +161,7 @@ func (t *transferQueueStandbyTaskExecutor) processActivityTask(
 }
 
 func (t *transferQueueStandbyTaskExecutor) processWorkflowTask(
+	ctx context.Context,
 	transferTask *persistencespb.TransferTaskInfo,
 ) error {
 
@@ -206,6 +210,7 @@ func (t *transferQueueStandbyTaskExecutor) processWorkflowTask(
 	}
 
 	return t.processTransfer(
+		ctx,
 		processTaskIfClosed,
 		transferTask,
 		actionFn,
@@ -221,6 +226,7 @@ func (t *transferQueueStandbyTaskExecutor) processWorkflowTask(
 }
 
 func (t *transferQueueStandbyTaskExecutor) processCloseExecution(
+	ctx context.Context,
 	transferTask *persistencespb.TransferTaskInfo,
 ) error {
 
@@ -278,6 +284,7 @@ func (t *transferQueueStandbyTaskExecutor) processCloseExecution(
 	}
 
 	return t.processTransfer(
+		ctx,
 		processTaskIfClosed,
 		transferTask,
 		actionFn,
@@ -286,6 +293,7 @@ func (t *transferQueueStandbyTaskExecutor) processCloseExecution(
 }
 
 func (t *transferQueueStandbyTaskExecutor) processCancelExecution(
+	ctx context.Context,
 	transferTask *persistencespb.TransferTaskInfo,
 ) error {
 
@@ -306,6 +314,7 @@ func (t *transferQueueStandbyTaskExecutor) processCancelExecution(
 	}
 
 	return t.processTransfer(
+		ctx,
 		processTaskIfClosed,
 		transferTask,
 		actionFn,
@@ -321,6 +330,7 @@ func (t *transferQueueStandbyTaskExecutor) processCancelExecution(
 }
 
 func (t *transferQueueStandbyTaskExecutor) processSignalExecution(
+	ctx context.Context,
 	transferTask *persistencespb.TransferTaskInfo,
 ) error {
 
@@ -341,6 +351,7 @@ func (t *transferQueueStandbyTaskExecutor) processSignalExecution(
 	}
 
 	return t.processTransfer(
+		ctx,
 		processTaskIfClosed,
 		transferTask,
 		actionFn,
@@ -356,6 +367,7 @@ func (t *transferQueueStandbyTaskExecutor) processSignalExecution(
 }
 
 func (t *transferQueueStandbyTaskExecutor) processStartChildExecution(
+	ctx context.Context,
 	transferTask *persistencespb.TransferTaskInfo,
 ) error {
 
@@ -380,6 +392,7 @@ func (t *transferQueueStandbyTaskExecutor) processStartChildExecution(
 	}
 
 	return t.processTransfer(
+		ctx,
 		processTaskIfClosed,
 		transferTask,
 		actionFn,
@@ -395,14 +408,16 @@ func (t *transferQueueStandbyTaskExecutor) processStartChildExecution(
 }
 
 func (t *transferQueueStandbyTaskExecutor) processTransfer(
+	ctx context.Context,
 	processTaskIfClosed bool,
 	taskInfo queueTaskInfo,
 	actionFn standbyActionFn,
 	postActionFn standbyPostActionFn,
 ) (retError error) {
+	var cancel context.CancelFunc
+	ctx, cancel = context.WithTimeout(ctx, taskTimeout)
 
 	transferTask := taskInfo.(*persistencespb.TransferTaskInfo)
-	ctx, cancel := context.WithTimeout(context.Background(), taskTimeout)
 	defer cancel()
 	namespaceID, execution := t.getNamespaceIDAndWorkflowExecution(transferTask)
 	context, release, err := t.cache.GetOrCreateWorkflowExecution(
