@@ -36,6 +36,7 @@ import (
 	historypb "go.temporal.io/api/history/v1"
 	"go.temporal.io/api/serviceerror"
 
+	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/cluster"
@@ -52,6 +53,8 @@ import (
 )
 
 type (
+	workflowResetReapplyEventsFn func(ctx context.Context, resetMutableState workflow.MutableState) error
+
 	workflowResetter interface {
 		// resetWorkflow is the new NDC compatible workflow reset logic
 		resetWorkflow(
@@ -78,9 +81,10 @@ type (
 		shard             shard.Context
 		namespaceCache    cache.NamespaceCache
 		clusterMetadata   cluster.Metadata
-		historyV2Mgr      persistence.HistoryManager
+		executionMgr      persistence.ExecutionManager
 		historyCache      *workflow.Cache
 		newStateRebuilder nDCStateRebuilderProvider
+		transaction       workflow.Transaction
 		logger            log.Logger
 	}
 )
@@ -96,12 +100,13 @@ func newWorkflowResetter(
 		shard:           shard,
 		namespaceCache:  shard.GetNamespaceCache(),
 		clusterMetadata: shard.GetClusterMetadata(),
-		historyV2Mgr:    shard.GetHistoryManager(),
+		executionMgr:    shard.GetExecutionManager(),
 		historyCache:    historyCache,
 		newStateRebuilder: func() nDCStateRebuilder {
 			return newNDCStateRebuilder(shard, logger)
 		},
-		logger: logger,
+		transaction: workflow.NewTransaction(shard),
+		logger:      logger,
 	}
 }
 
@@ -128,8 +133,10 @@ func (r *workflowResetterImpl) resetWorkflow(
 	}
 	resetWorkflowVersion := namespaceEntry.GetFailoverVersion()
 
+	var currentWorkflowMutation *persistence.WorkflowMutation
+	var currentWorkflowEventsSeq []*persistence.WorkflowEvents
+	var reapplyEventsFn workflowResetReapplyEventsFn
 	currentMutableState := currentWorkflow.getMutableState()
-	currentWorkflowTerminated := false
 	if currentMutableState.IsWorkflowExecutionRunning() {
 		if err := r.terminateWorkflow(
 			currentMutableState,
@@ -138,7 +145,56 @@ func (r *workflowResetterImpl) resetWorkflow(
 			return err
 		}
 		resetWorkflowVersion = currentMutableState.GetCurrentVersion()
-		currentWorkflowTerminated = true
+
+		currentWorkflowMutation, currentWorkflowEventsSeq, err = currentMutableState.CloseTransactionAsMutation(
+			r.shard.GetTimeSource().Now(),
+			workflow.TransactionPolicyActive,
+		)
+		if err != nil {
+			return err
+		}
+		currentWorkflowMutation.ExecutionInfo.ExecutionStats = &persistencespb.ExecutionStats{
+			HistorySize: currentWorkflow.getContext().GetHistorySize(),
+		}
+
+		reapplyEventsFn = func(ctx context.Context, resetMutableState workflow.MutableState) error {
+			lastVisitedRunID, err := r.reapplyContinueAsNewWorkflowEvents(
+				ctx,
+				resetMutableState,
+				namespaceID,
+				workflowID,
+				baseRunID,
+				baseBranchToken,
+				baseRebuildLastEventID+1,
+				baseNextEventID,
+			)
+			if err != nil {
+				return err
+			}
+
+			if lastVisitedRunID == currentMutableState.GetExecutionState().RunId {
+				for _, event := range currentWorkflowEventsSeq {
+					if err := r.reapplyEvents(resetMutableState, event.Events); err != nil {
+						return err
+					}
+				}
+			}
+			return nil
+		}
+	} else {
+		reapplyEventsFn = func(ctx context.Context, resetMutableState workflow.MutableState) error {
+			_, err := r.reapplyContinueAsNewWorkflowEvents(
+				ctx,
+				resetMutableState,
+				namespaceID,
+				workflowID,
+				baseRunID,
+				baseBranchToken,
+				baseRebuildLastEventID+1,
+				baseNextEventID,
+			)
+			return err
+		}
 	}
 
 	resetWorkflow, err := r.prepareResetWorkflow(
@@ -149,22 +205,34 @@ func (r *workflowResetterImpl) resetWorkflow(
 		baseBranchToken,
 		baseRebuildLastEventID,
 		baseRebuildLastEventVersion,
-		baseNextEventID,
 		resetRunID,
 		resetRequestID,
 		resetWorkflowVersion,
 		resetReason,
-		additionalReapplyEvents,
-		resetReapplyType,
 	)
 	if err != nil {
 		return err
 	}
 	defer resetWorkflow.getReleaseFn()(retError)
 
+	if err := r.reapplyEventsToResetWorkflow(
+		ctx,
+		resetWorkflow.getMutableState(),
+		resetReapplyType,
+		reapplyEventsFn,
+		additionalReapplyEvents,
+	); err != nil {
+		return err
+	}
+
+	if err := workflow.ScheduleWorkflowTask(resetWorkflow.getMutableState()); err != nil {
+		return err
+	}
+
 	return r.persistToDB(
-		currentWorkflowTerminated,
 		currentWorkflow,
+		currentWorkflowMutation,
+		currentWorkflowEventsSeq,
 		resetWorkflow,
 	)
 }
@@ -177,13 +245,10 @@ func (r *workflowResetterImpl) prepareResetWorkflow(
 	baseBranchToken []byte,
 	baseRebuildLastEventID int64,
 	baseRebuildLastEventVersion int64,
-	baseNextEventID int64,
 	resetRunID string,
 	resetRequestID string,
 	resetWorkflowVersion int64,
 	resetReason string,
-	additionalReapplyEvents []*historypb.HistoryEvent,
-	resetReapplyType enumspb.ResetReapplyType,
 ) (nDCWorkflow, error) {
 
 	resetWorkflow, err := r.replayResetWorkflow(
@@ -243,19 +308,23 @@ func (r *workflowResetterImpl) prepareResetWorkflow(
 		return nil, err
 	}
 
+	return resetWorkflow, nil
+}
+
+func (r *workflowResetterImpl) reapplyEventsToResetWorkflow(
+	ctx context.Context,
+	resetMutableState workflow.MutableState,
+	resetReapplyType enumspb.ResetReapplyType,
+	reapplyEventsApplier workflowResetReapplyEventsFn,
+	additionalReapplyEvents []*historypb.HistoryEvent,
+) error {
 	switch resetReapplyType {
 	case enumspb.RESET_REAPPLY_TYPE_SIGNAL:
-		if err := r.reapplyContinueAsNewWorkflowEvents(
+		if err := reapplyEventsApplier(
 			ctx,
 			resetMutableState,
-			namespaceID,
-			workflowID,
-			baseRunID,
-			baseBranchToken,
-			baseRebuildLastEventID+1,
-			baseNextEventID,
 		); err != nil {
-			return nil, err
+			return err
 		}
 	case enumspb.RESET_REAPPLY_TYPE_NONE:
 		// noop
@@ -264,37 +333,19 @@ func (r *workflowResetterImpl) prepareResetWorkflow(
 	}
 
 	if err := r.reapplyEvents(resetMutableState, additionalReapplyEvents); err != nil {
-		return nil, err
+		return err
 	}
 
-	if err := workflow.ScheduleWorkflowTask(resetMutableState); err != nil {
-		return nil, err
-	}
-
-	return resetWorkflow, nil
+	return nil
 }
 
 func (r *workflowResetterImpl) persistToDB(
-	currentWorkflowTerminated bool,
 	currentWorkflow nDCWorkflow,
+	currentWorkflowMutation *persistence.WorkflowMutation,
+	currentWorkflowEventsSeq []*persistence.WorkflowEvents,
 	resetWorkflow nDCWorkflow,
 ) error {
 
-	if currentWorkflowTerminated {
-		return currentWorkflow.getContext().UpdateWorkflowExecutionWithNewAsActive(
-			r.shard.GetTimeSource().Now(),
-			resetWorkflow.getContext(),
-			resetWorkflow.getMutableState(),
-		)
-	}
-
-	currentMutableState := currentWorkflow.getMutableState()
-	currentRunID := currentMutableState.GetExecutionState().GetRunId()
-	currentLastWriteVersion, err := currentMutableState.GetLastWriteVersion()
-	if err != nil {
-		return err
-	}
-
 	now := r.shard.GetTimeSource().Now()
 	resetWorkflowSnapshot, resetWorkflowEventsSeq, err := resetWorkflow.getMutableState().CloseTransactionAsSnapshot(
 		now,
@@ -303,14 +354,33 @@ func (r *workflowResetterImpl) persistToDB(
 	if err != nil {
 		return err
 	}
-	var resetHistorySize int64
-	for _, workflowEvents := range resetWorkflowEventsSeq {
-		size, err := resetWorkflow.getContext().PersistNonFirstWorkflowEvents(workflowEvents)
-		if err != nil {
-			return err
-		}
-		resetHistorySize += size
+	resetWorkflowSnapshot.ExecutionInfo.ExecutionStats = &persistencespb.ExecutionStats{
+		HistorySize: resetWorkflow.getContext().GetHistorySize(),
 	}
+
+	if currentWorkflowMutation != nil {
+		if currentWorkflowSizeDiff, resetWorkflowSizeDiff, err := r.transaction.UpdateWorkflowExecution(
+			persistence.UpdateWorkflowModeUpdateCurrent,
+			currentWorkflowMutation,
+			currentWorkflowEventsSeq,
+			resetWorkflowSnapshot,
+			resetWorkflowEventsSeq,
+		); err != nil {
+			return err
+		} else {
+			currentWorkflow.getContext().SetHistorySize(currentWorkflow.getContext().GetHistorySize() + currentWorkflowSizeDiff)
+			resetWorkflow.getContext().SetHistorySize(resetWorkflow.getContext().GetHistorySize() + resetWorkflowSizeDiff)
+		}
+		return nil
+	}
+
+	currentMutableState := currentWorkflow.getMutableState()
+	currentRunID := currentMutableState.GetExecutionState().GetRunId()
+	currentLastWriteVersion, err := currentMutableState.GetLastWriteVersion()
+	if err != nil {
+		return err
+	}
+
 	return resetWorkflow.getContext().CreateWorkflowExecution(
 		now,
 		persistence.CreateWorkflowModeContinueAsNew,
@@ -318,7 +388,7 @@ func (r *workflowResetterImpl) persistToDB(
 		currentLastWriteVersion,
 		resetWorkflow.getMutableState(),
 		resetWorkflowSnapshot,
-		resetHistorySize,
+		resetWorkflowEventsSeq,
 	)
 }
 
@@ -352,7 +422,6 @@ func (r *workflowResetterImpl) replayResetWorkflow(
 			RunId:      resetRunID,
 		},
 		r.shard,
-		r.shard.GetExecutionManager(),
 		r.logger,
 	)
 	resetMutableState, resetHistorySize, err := r.newStateRebuilder().rebuild(
@@ -482,7 +551,7 @@ func (r *workflowResetterImpl) forkAndGenerateBranchToken(
 ) ([]byte, error) {
 	// fork a new history branch
 	shardID := r.shard.GetShardID()
-	resp, err := r.historyV2Mgr.ForkHistoryBranch(&persistence.ForkHistoryBranchRequest{
+	resp, err := r.executionMgr.ForkHistoryBranch(&persistence.ForkHistoryBranchRequest{
 		ForkBranchToken: forkBranchToken,
 		ForkNodeID:      forkNodeID,
 		Info:            persistence.BuildHistoryGarbageCleanupInfo(namespaceID, workflowID, resetRunID),
@@ -519,11 +588,13 @@ func (r *workflowResetterImpl) reapplyContinueAsNewWorkflowEvents(
 	baseBranchToken []byte,
 	baseRebuildNextEventID int64,
 	baseNextEventID int64,
-) error {
+) (string, error) {
 
 	// TODO change this logic to fetching all workflow [baseWorkflow, currentWorkflow]
 	//  from visibility for better coverage of events eligible for re-application.
 
+	lastVisitedRunID := baseRunID
+
 	// first special handling the remaining events for base workflow
 	nextRunID, err := r.reapplyWorkflowEvents(
 		resetMutableState,
@@ -537,9 +608,9 @@ func (r *workflowResetterImpl) reapplyContinueAsNewWorkflowEvents(
 	case *serviceerror.DataLoss:
 		// log event
 		r.logger.Error("encounter data loss event", tag.WorkflowNamespaceID(namespaceID), tag.WorkflowID(workflowID), tag.WorkflowRunID(baseRunID))
-		return err
+		return "", err
 	default:
-		return err
+		return "", err
 	}
 
 	getNextEventIDBranchToken := func(runID string) (nextEventID int64, branchToken []byte, retError error) {
@@ -573,9 +644,10 @@ func (r *workflowResetterImpl) reapplyContinueAsNewWorkflowEvents(
 
 	// second for remaining continue as new workflow, reapply eligible events
 	for len(nextRunID) != 0 {
+		lastVisitedRunID = nextRunID
 		nextWorkflowNextEventID, nextWorkflowBranchToken, err := getNextEventIDBranchToken(nextRunID)
 		if err != nil {
-			return err
+			return "", err
 		}
 
 		nextRunID, err = r.reapplyWorkflowEvents(
@@ -589,13 +661,13 @@ func (r *workflowResetterImpl) reapplyContinueAsNewWorkflowEvents(
 			// noop
 		case *serviceerror.DataLoss:
 			// log event
-			r.logger.Error("encounter data loss event", tag.WorkflowNamespaceID(namespaceID), tag.WorkflowID(workflowID), tag.WorkflowRunID(baseRunID))
-			return err
+			r.logger.Error("encounter data loss event", tag.WorkflowNamespaceID(namespaceID), tag.WorkflowID(workflowID), tag.WorkflowRunID(nextRunID))
+			return "", err
 		default:
-			return err
+			return "", err
 		}
 	}
-	return nil
+	return lastVisitedRunID, nil
 }
 
 func (r *workflowResetterImpl) reapplyWorkflowEvents(
@@ -669,7 +741,7 @@ func (r *workflowResetterImpl) getPaginationFn(
 
 	return func(paginationToken []byte) ([]interface{}, []byte, error) {
 
-		resp, err := r.historyV2Mgr.ReadHistoryBranchByBatch(&persistence.ReadHistoryBranchRequest{
+		resp, err := r.executionMgr.ReadHistoryBranchByBatch(&persistence.ReadHistoryBranchRequest{
 			BranchToken:   branchToken,
 			MinEventID:    firstEventID,
 			MaxEventID:    nextEventID,
