@@ -217,10 +217,15 @@ func (e *matchingEngineImpl) updateTaskQueue(taskQueue *taskQueueID, mgr taskQue
 	e.taskQueues[*taskQueue] = mgr
 }
 
-func (e *matchingEngineImpl) removeTaskQueueManager(id *taskQueueID) {
+func (e *matchingEngineImpl) removeTaskQueueManager(delTQM taskQueueManager) {
+	queueID := delTQM.QueueID()
 	e.taskQueuesLock.Lock()
 	defer e.taskQueuesLock.Unlock()
-	delete(e.taskQueues, *id)
+	foundTQM, ok := e.taskQueues[*queueID]
+	if !ok || foundTQM != delTQM {
+		return
+	}
+	delete(e.taskQueues, *queueID)
 }
 
 // AddWorkflowTask either delivers task directly to waiting poller or save it into task queue persistence.
@@ -702,16 +707,17 @@ func (e *matchingEngineImpl) getTask(
 	return tlMgr.GetTask(ctx, maxDispatchPerSecond)
 }
 
-func (e *matchingEngineImpl) unloadTaskQueue(id *taskQueueID) {
+func (e *matchingEngineImpl) unloadTaskQueue(unloadTQM taskQueueManager) {
+	queueID := unloadTQM.QueueID()
 	e.taskQueuesLock.Lock()
-	tlMgr, ok := e.taskQueues[*id]
-	if ok {
-		delete(e.taskQueues, *id)
+	foundTQM, ok := e.taskQueues[*queueID]
+	if !ok || foundTQM != unloadTQM {
+		e.taskQueuesLock.Unlock()
+		return
 	}
+	delete(e.taskQueues, *queueID)
 	e.taskQueuesLock.Unlock()
-	if ok {
-		tlMgr.Stop()
-	}
+	foundTQM.Stop()
 }
 
 // Populate the workflow task response based on context and scheduled/started events.
