@@ -26,6 +26,8 @@ package matching
 
 import (
 	"context"
+	"errors"
+	"math"
 	"sync"
 	"sync/atomic"
 	"testing"
@@ -34,23 +36,28 @@ import (
 	"github.com/golang/mock/gomock"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
+	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
 
+	enumsspb "go.temporal.io/server/api/enums/v1"
 	persistencespb "go.temporal.io/server/api/persistence/v1"
+	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/dynamicconfig"
 	"go.temporal.io/server/common/log"
-	"go.temporal.io/server/common/log/tag"
+	"go.temporal.io/server/common/persistence"
 	"go.temporal.io/server/common/primitives/timestamp"
 )
 
+var rpsInf = math.Inf(1)
+
 func TestDeliverBufferTasks(t *testing.T) {
 	controller := gomock.NewController(t)
 	defer controller.Finish()
 
 	tests := []func(tlm *taskQueueManagerImpl){
 		func(tlm *taskQueueManagerImpl) { close(tlm.taskReader.taskBuffer) },
-		func(tlm *taskQueueManagerImpl) { close(tlm.taskReader.dispatcherShutdownC) },
+		func(tlm *taskQueueManagerImpl) { close(tlm.taskReader.shutdownChan) },
 		func(tlm *taskQueueManagerImpl) {
 			rps := 0.1
 			tlm.matcher.UpdateRatelimit(&rps)
@@ -61,7 +68,7 @@ func TestDeliverBufferTasks(t *testing.T) {
 		},
 	}
 	for _, test := range tests {
-		tlm := createTestTaskQueueManager(controller)
+		tlm := mustCreateTestTaskQueueManager(t, controller)
 		var wg sync.WaitGroup
 		wg.Add(1)
 		go func() {
@@ -78,7 +85,7 @@ func TestDeliverBufferTasks_NoPollers(t *testing.T) {
 	controller := gomock.NewController(t)
 	defer controller.Finish()
 
-	tlm := createTestTaskQueueManager(controller)
+	tlm := mustCreateTestTaskQueueManager(t, controller)
 	tlm.taskReader.taskBuffer <- &persistencespb.AllocatedTaskInfo{}
 	var wg sync.WaitGroup
 	wg.Add(1)
@@ -95,7 +102,7 @@ func TestReadLevelForAllExpiredTasksInBatch(t *testing.T) {
 	controller := gomock.NewController(t)
 	defer controller.Finish()
 
-	tlm := createTestTaskQueueManager(controller)
+	tlm := mustCreateTestTaskQueueManager(t, controller)
 	tlm.db.rangeID = int64(1)
 	tlm.db.ackLevel = int64(0)
 	tlm.taskAckManager.setAckLevel(tlm.db.ackLevel)
@@ -121,7 +128,7 @@ func TestReadLevelForAllExpiredTasksInBatch(t *testing.T) {
 		},
 	}
 
-	require.True(t, tlm.taskReader.addTasksToBuffer(tasks, time.Now().UTC(), time.NewTimer(time.Minute)))
+	require.True(t, tlm.taskReader.addTasksToBuffer(tasks))
 	require.Equal(t, int64(0), tlm.taskAckManager.getAckLevel())
 	require.Equal(t, int64(12), tlm.taskAckManager.getReadLevel())
 
@@ -141,16 +148,146 @@ func TestReadLevelForAllExpiredTasksInBatch(t *testing.T) {
 			},
 			TaskId: 14,
 		},
-	}, time.Now().UTC(), time.NewTimer(time.Minute)))
+	}))
 	require.Equal(t, int64(0), tlm.taskAckManager.getAckLevel())
 	require.Equal(t, int64(14), tlm.taskAckManager.getReadLevel())
 }
 
-func createTestTaskQueueManager(controller *gomock.Controller) *taskQueueManagerImpl {
-	return createTestTaskQueueManagerWithConfig(controller, defaultTestConfig())
+type testIDBlockAlloc struct {
+	rid   int64
+	alloc func() (taskQueueState, error)
 }
 
-func createTestTaskQueueManagerWithConfig(controller *gomock.Controller, cfg *Config) *taskQueueManagerImpl {
+func (a *testIDBlockAlloc) RangeID() int64 {
+	return a.rid
+}
+
+func (a *testIDBlockAlloc) RenewLease() (taskQueueState, error) {
+	s, err := a.alloc()
+	if err == nil {
+		a.rid = s.rangeID
+	}
+	return s, err
+}
+
+func makeTestBlocAlloc(f func() (taskQueueState, error)) taskQueueManagerOpt {
+	return withIDBlockAllocator(&testIDBlockAlloc{alloc: f})
+}
+
+func TestSyncMatchLeasingUnavailable(t *testing.T) {
+	cfg := NewConfig(dynamicconfig.NewNoopCollection())
+	cfg.ResilientSyncMatch = func(...dynamicconfig.FilterOption) bool { return true }
+	tqm := mustCreateTestTaskQueueManagerWithConfig(t, gomock.NewController(t), cfg,
+		makeTestBlocAlloc(func() (taskQueueState, error) {
+			// any error other than ConditionFailedError indicates an
+			// availability problem at a lower layer so the TQM should NOT
+			// unload itself because resilient sync match is enabled.
+			return taskQueueState{}, errors.New(t.Name())
+		}))
+	pollctx, stoppoller := context.WithCancel(context.Background())
+	defer stoppoller()
+	go func() {
+		task, err := tqm.GetTask(pollctx, &rpsInf)
+		if errors.Is(err, ErrNoTasks) {
+			return
+		}
+		task.finish(err)
+	}()
+	// tqm.GetTask() needs some time to attach the goro started above to the
+	// internal task channel. Sorry for this but it appears unavoidable.
+	time.Sleep(10 * time.Millisecond)
+	tqm.Start()
+	defer tqm.Stop()
+
+	sync, err := tqm.AddTask(context.TODO(), addTaskParams{
+		execution: &commonpb.WorkflowExecution{},
+		taskInfo:  &persistencespb.TaskInfo{},
+		source:    enumsspb.TASK_SOURCE_HISTORY})
+	require.NoError(t, err)
+	require.True(t, sync)
+}
+
+func TestForeignPartitionOwnerCausesUnload(t *testing.T) {
+	cc := dynamicconfig.NewMutableEphemeralClient(
+		dynamicconfig.Set(dynamicconfig.ResilientSyncMatch, true))
+	cfg := NewConfig(dynamicconfig.NewCollection(cc, log.NewTestLogger()))
+	cfg.RangeSize = 1 // TaskID block size
+	var leaseErr error = nil
+	tqm := mustCreateTestTaskQueueManagerWithConfig(t, gomock.NewController(t), cfg,
+		makeTestBlocAlloc(func() (taskQueueState, error) {
+			return taskQueueState{rangeID: 1}, leaseErr
+		}))
+	tqm.Start()
+	defer tqm.Stop()
+
+	// TQM started succesfully with an ID block of size 1. Perform one send
+	// without a poller to consume the one task ID from the reserved block.
+	sync, err := tqm.AddTask(context.TODO(), addTaskParams{
+		execution: &commonpb.WorkflowExecution{},
+		taskInfo:  &persistencespb.TaskInfo{},
+		source:    enumsspb.TASK_SOURCE_HISTORY})
+	require.False(t, sync)
+	require.NoError(t, err)
+
+	// TQM's ID block should be empty so the next AddTask will trigger an
+	// attempt to obtain more IDs. This specific error type indicates that
+	// another service instance has become the owner of the partition
+	leaseErr = &persistence.ConditionFailedError{Msg: "should kill the tqm"}
+	require.True(t, errIndicatesForeignLessee(leaseErr))
+
+	sync, err = tqm.AddTask(context.TODO(), addTaskParams{
+		execution: &commonpb.WorkflowExecution{},
+		taskInfo:  &persistencespb.TaskInfo{},
+		source:    enumsspb.TASK_SOURCE_HISTORY,
+	})
+	require.False(t, sync)
+	require.ErrorIs(t, err, errShutdown)
+}
+
+func TestAnyErrorCausesUnloadWhenResilientSyncMatchDisabled(t *testing.T) {
+	cc := dynamicconfig.NewMutableEphemeralClient(
+		dynamicconfig.Set(dynamicconfig.ResilientSyncMatch, true))
+	cfg := NewConfig(dynamicconfig.NewCollection(cc, log.NewTestLogger()))
+	tqm := mustCreateTestTaskQueueManagerWithConfig(t, gomock.NewController(t), cfg,
+		makeTestBlocAlloc(func() (taskQueueState, error) {
+			return taskQueueState{}, errors.New("any error will do")
+		}))
+	tqm.Start()
+	defer tqm.Stop()
+	_, err := tqm.AddTask(context.TODO(), addTaskParams{
+		execution: &commonpb.WorkflowExecution{},
+		taskInfo:  &persistencespb.TaskInfo{},
+		source:    enumsspb.TASK_SOURCE_HISTORY,
+	})
+	require.Error(t, err)
+}
+
+func mustCreateTestTaskQueueManager(
+	t *testing.T,
+	controller *gomock.Controller,
+	opts ...taskQueueManagerOpt,
+) *taskQueueManagerImpl {
+	t.Helper()
+	return mustCreateTestTaskQueueManagerWithConfig(t, controller, defaultTestConfig(), opts...)
+}
+
+func mustCreateTestTaskQueueManagerWithConfig(
+	t *testing.T,
+	controller *gomock.Controller,
+	cfg *Config,
+	opts ...taskQueueManagerOpt,
+) *taskQueueManagerImpl {
+	t.Helper()
+	tqm, err := createTestTaskQueueManagerWithConfig(controller, cfg, opts...)
+	require.NoError(t, err)
+	return tqm
+}
+
+func createTestTaskQueueManagerWithConfig(
+	controller *gomock.Controller,
+	cfg *Config,
+	opts ...taskQueueManagerOpt,
+) (*taskQueueManagerImpl, error) {
 	logger := log.NewTestLogger()
 	tm := newTestTaskManager(logger)
 	mockNamespaceCache := cache.NewMockNamespaceCache(controller)
@@ -162,18 +299,19 @@ func createTestTaskQueueManagerWithConfig(controller *gomock.Controller, cfg *Co
 	dID := "deadbeef-0000-4567-890a-bcdef0123456"
 	tlID := newTestTaskQueueID(dID, tl, enumspb.TASK_QUEUE_TYPE_ACTIVITY)
 	tlKind := enumspb.TASK_QUEUE_KIND_NORMAL
-	tlMgr, err := newTaskQueueManager(me, tlID, tlKind, cfg)
+	tlMgr, err := newTaskQueueManager(me, tlID, tlKind, cfg, opts...)
 	if err != nil {
-		logger.Fatal("error when createTestTaskQueueManager", tag.Error(err))
+		return nil, err
 	}
-	return tlMgr.(*taskQueueManagerImpl)
+	me.taskQueues[*tlID] = tlMgr
+	return tlMgr.(*taskQueueManagerImpl), nil
 }
 
 func TestIsTaskAddedRecently(t *testing.T) {
 	controller := gomock.NewController(t)
 	defer controller.Finish()
 
-	tlm := createTestTaskQueueManager(controller)
+	tlm := mustCreateTestTaskQueueManager(t, controller)
 	require.True(t, tlm.taskReader.isTaskAddedRecently(time.Now().UTC()))
 	require.False(t, tlm.taskReader.isTaskAddedRecently(time.Now().UTC().Add(-tlm.config.MaxTaskqueueIdleTime())))
 	require.True(t, tlm.taskReader.isTaskAddedRecently(time.Now().UTC().Add(1*time.Second)))
@@ -189,7 +327,7 @@ func TestDescribeTaskQueue(t *testing.T) {
 	PollerIdentity := "test-poll"
 
 	// Create taskQueue Manager and set taskQueue state
-	tlm := createTestTaskQueueManager(controller)
+	tlm := mustCreateTestTaskQueueManager(t, controller)
 	tlm.db.rangeID = int64(1)
 	tlm.db.ackLevel = int64(0)
 	tlm.taskAckManager.setAckLevel(tlm.db.ackLevel)
@@ -238,8 +376,6 @@ func TestDescribeTaskQueue(t *testing.T) {
 }
 
 func tlMgrStartWithoutNotifyEvent(tlm *taskQueueManagerImpl) {
-	// mimic tlm.Start() but avoid calling notifyEvent
-	tlm.startWG.Done()
 	go tlm.taskReader.dispatchBufferedTasks()
 	go tlm.taskReader.getTasksPump()
 }
@@ -249,31 +385,34 @@ func TestCheckIdleTaskQueue(t *testing.T) {
 	defer controller.Finish()
 
 	cfg := NewConfig(dynamicconfig.NewNoopCollection())
-	cfg.IdleTaskqueueCheckInterval = dynamicconfig.GetDurationPropertyFnFilteredByTaskQueueInfo(10 * time.Millisecond)
+	cfg.IdleTaskqueueCheckInterval = dynamicconfig.GetDurationPropertyFnFilteredByTaskQueueInfo(2 * time.Second)
 
 	// Idle
-	tlm := createTestTaskQueueManagerWithConfig(controller, cfg)
+	tlm := mustCreateTestTaskQueueManagerWithConfig(t, controller, cfg)
+	tlm.Start()
 	tlMgrStartWithoutNotifyEvent(tlm)
-	time.Sleep(20 * time.Millisecond)
-	require.False(t, atomic.CompareAndSwapInt32(&tlm.stopped, 0, 1))
+	time.Sleep(1 * time.Second)
+	require.Equal(t, common.DaemonStatusStarted, atomic.LoadInt32(&tlm.status))
 
 	// Active poll-er
-	tlm = createTestTaskQueueManagerWithConfig(controller, cfg)
+	tlm = mustCreateTestTaskQueueManagerWithConfig(t, controller, cfg)
+	tlm.Start()
 	tlm.pollerHistory.updatePollerInfo(pollerIdentity("test-poll"), nil)
 	require.Equal(t, 1, len(tlm.GetAllPollerInfo()))
 	tlMgrStartWithoutNotifyEvent(tlm)
-	time.Sleep(20 * time.Millisecond)
-	require.Equal(t, int32(0), tlm.stopped)
+	time.Sleep(1 * time.Second)
+	require.Equal(t, common.DaemonStatusStarted, atomic.LoadInt32(&tlm.status))
 	tlm.Stop()
-	require.Equal(t, int32(1), tlm.stopped)
+	require.Equal(t, common.DaemonStatusStopped, atomic.LoadInt32(&tlm.status))
 
 	// Active adding task
-	tlm = createTestTaskQueueManagerWithConfig(controller, cfg)
+	tlm = mustCreateTestTaskQueueManagerWithConfig(t, controller, cfg)
+	tlm.Start()
 	require.Equal(t, 0, len(tlm.GetAllPollerInfo()))
 	tlMgrStartWithoutNotifyEvent(tlm)
 	tlm.taskReader.Signal()
-	time.Sleep(20 * time.Millisecond)
-	require.Equal(t, int32(0), tlm.stopped)
+	time.Sleep(1 * time.Second)
+	require.Equal(t, common.DaemonStatusStarted, atomic.LoadInt32(&tlm.status))
 	tlm.Stop()
-	require.Equal(t, int32(1), tlm.stopped)
+	require.Equal(t, common.DaemonStatusStopped, atomic.LoadInt32(&tlm.status))
 }
