@@ -36,7 +36,7 @@ import (
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
 	"go.temporal.io/server/common/metrics"
-	esclient "go.temporal.io/server/common/persistence/elasticsearch/client"
+	"go.temporal.io/server/common/persistence/visibility/elasticsearch/client"
 	"go.temporal.io/server/common/searchattribute"
 )
 
@@ -54,10 +54,12 @@ type (
 		IndexName string
 		// Search attributes that need to be added to the index.
 		CustomAttributesToAdd map[string]enumspb.IndexedValueType
+		// If true skip Elasticsearch schema update and only update cluster metadata.
+		SkipSchemaUpdate bool
 	}
 
 	activities struct {
-		esClient      esclient.Client
+		esClient      client.Client
 		saManager     searchattribute.Manager
 		metricsClient metrics.Client
 		logger        log.Logger
@@ -96,7 +98,7 @@ var (
 )
 
 func newActivities(
-	esClient esclient.Client,
+	esClient client.Client,
 	saManager searchattribute.Manager,
 	metricsClient metrics.Client,
 	logger log.Logger,
@@ -115,17 +117,20 @@ func AddSearchAttributesWorkflow(ctx workflow.Context, params WorkflowParams) er
 	logger.Info("Workflow started.", "wf-type", WorkflowName)
 
 	var a *activities
+	var err error
 
-	ctx1 := workflow.WithActivityOptions(ctx, addESMappingFieldActivityOptions)
-	err := workflow.ExecuteActivity(ctx1, a.AddESMappingFieldActivity, params).Get(ctx, nil)
-	if err != nil {
-		return fmt.Errorf("%w: AddESMappingFieldActivity: %v", ErrUnableToExecuteActivity, err)
-	}
+	if !params.SkipSchemaUpdate {
+		ctx1 := workflow.WithActivityOptions(ctx, addESMappingFieldActivityOptions)
+		err = workflow.ExecuteActivity(ctx1, a.AddESMappingFieldActivity, params).Get(ctx, nil)
+		if err != nil {
+			return fmt.Errorf("%w: AddESMappingFieldActivity: %v", ErrUnableToExecuteActivity, err)
+		}
 
-	ctx2 := workflow.WithActivityOptions(ctx, waitForYellowStatusActivityOptions)
-	err = workflow.ExecuteActivity(ctx2, a.WaitForYellowStatusActivity, params.IndexName).Get(ctx, nil)
-	if err != nil {
-		return fmt.Errorf("%w: WaitForYellowStatusActivity: %v", ErrUnableToExecuteActivity, err)
+		ctx2 := workflow.WithActivityOptions(ctx, waitForYellowStatusActivityOptions)
+		err = workflow.ExecuteActivity(ctx2, a.WaitForYellowStatusActivity, params.IndexName).Get(ctx, nil)
+		if err != nil {
+			return fmt.Errorf("%w: WaitForYellowStatusActivity: %v", ErrUnableToExecuteActivity, err)
+		}
 	}
 
 	ctx3 := workflow.WithActivityOptions(ctx, updateClusterMetadataActivityOptions)
@@ -148,7 +153,7 @@ func (a *activities) AddESMappingFieldActivity(ctx context.Context, params Workf
 	_, err := a.esClient.PutMapping(ctx, params.IndexName, params.CustomAttributesToAdd)
 	if err != nil {
 		a.metricsClient.IncCounter(metrics.AddSearchAttributesWorkflowScope, metrics.AddSearchAttributesFailuresCount)
-		if esclient.IsRetryableError(err) {
+		if client.IsRetryableError(err) {
 			a.logger.Error("Unable to update Elasticsearch mapping (retryable error).", tag.ESIndex(params.IndexName), tag.Error(err))
 			return fmt.Errorf("%w: %v", ErrUnableToUpdateESMapping, err)
 		}
@@ -162,7 +167,7 @@ func (a *activities) AddESMappingFieldActivity(ctx context.Context, params Workf
 
 func (a *activities) WaitForYellowStatusActivity(ctx context.Context, indexName string) error {
 	if a.esClient == nil {
-		a.logger.Info("Elasticsearch client is not configures. Skipping Elasticsearch status check.")
+		a.logger.Info("Elasticsearch client is not configured. Skipping Elasticsearch status check.")
 		return nil
 	}
 
