@@ -36,10 +36,10 @@ import (
 	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
 	"go.temporal.io/api/serviceerror"
-
 	enumsspb "go.temporal.io/server/api/enums/v1"
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
+	"go.temporal.io/server/common/dynamicconfig"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/persistence"
 	cassp "go.temporal.io/server/common/persistence/cassandra"
@@ -236,7 +236,6 @@ func AdminDBScan(c *cli.Context) {
 	rateLimiter := getRateLimiter(startingRPS, targetRPS)
 	session := connectToCassandra(c)
 	defer session.Close()
-	historyStore := cassp.NewHistoryV2PersistenceFromSession(session, log.NewNoopLogger())
 	scanOutputDirectories := createScanOutputDirectories()
 
 	reports := make(chan *ShardScanReport)
@@ -250,8 +249,7 @@ func AdminDBScan(c *cli.Context) {
 						scanOutputDirectories,
 						rateLimiter,
 						executionsPageSize,
-						payloadSerializer,
-						historyStore)
+						payloadSerializer)
 				}
 			}
 		}(i)
@@ -279,7 +277,6 @@ func scanShard(
 	limiter quotas.RateLimiter,
 	executionsPageSize int,
 	payloadSerializer serialization.Serializer,
-	historyStore persistence.HistoryStore,
 ) *ShardScanReport {
 	outputFiles, closeFn := createShardScanOutputFiles(shardID, scanOutputDirectories)
 	report := &ShardScanReport{
@@ -294,21 +291,15 @@ func scanShard(
 		deleteEmptyFiles(outputFiles.CorruptedExecutionFile, outputFiles.ExecutionCheckFailureFile, outputFiles.ShardScanReportFile)
 		closeFn()
 	}()
-	execStore, err := cassp.NewWorkflowExecutionPersistence(shardID, session, log.NewNoopLogger())
-	if err != nil {
-		report.Failure = &ShardScanReportFailure{
-			Note:    "failed to create execution store",
-			Details: err.Error(),
-		}
-		return report
-	}
-	execMan := persistence.NewExecutionManager(execStore, log.NewNoopLogger())
+	workflowStore := cassp.NewExecutionStore(session, log.NewNoopLogger())
+	execMan := persistence.NewExecutionManager(workflowStore, log.NewNoopLogger(), dynamicconfig.GetIntPropertyFn(common.DefaultTransactionSizeLimit))
 
 	var token []byte
 	isFirstIteration := true
 	for isFirstIteration || len(token) != 0 {
 		isFirstIteration = false
 		req := &persistence.ListConcreteExecutionsRequest{
+			ShardID:   shardID,
 			PageSize:  executionsPageSize,
 			PageToken: token,
 		}
@@ -335,9 +326,8 @@ func scanShard(
 				checkFailureWriter,
 				shardID,
 				limiter,
-				historyStore,
+				workflowStore,
 				&report.TotalDBRequests,
-				execStore,
 			)
 			switch historyVerificationResult {
 			case VerificationResultNoCorruption:
@@ -440,9 +430,8 @@ func fetchAndVerifyHistoryExists(
 	checkFailureWriter BufferedWriter,
 	shardID int32,
 	limiter quotas.RateLimiter,
-	historyStore persistence.HistoryStore,
+	executionStore persistence.ExecutionStore,
 	totalDBRequests *int64,
-	execStore persistence.ExecutionStore,
 ) (VerificationResult, *persistence.InternalReadHistoryBranchResponse, *persistencespb.HistoryBranch) {
 	var branch *persistencespb.HistoryBranch
 	currentVersionHistory, err := versionhistory.GetCurrentVersionHistory(executionInfo.VersionHistories)
@@ -473,9 +462,9 @@ func fetchAndVerifyHistoryExists(
 		PageSize:  historyPageSize,
 	}
 	preconditionForDBCall(totalDBRequests, limiter)
-	history, err := historyStore.ReadHistoryBranch(readHistoryBranchReq)
+	history, err := executionStore.ReadHistoryBranch(readHistoryBranchReq)
 
-	ecf, stillExists := concreteExecutionStillExists(executionInfo, executionState, shardID, execStore, limiter, totalDBRequests)
+	ecf, stillExists := concreteExecutionStillExists(executionInfo, executionState, shardID, executionStore, limiter, totalDBRequests)
 	if ecf != nil {
 		checkFailureWriter.Add(ecf)
 		return VerificationResultCheckFailure, nil, nil
@@ -652,6 +641,7 @@ func verifyCurrentExecution(
 		return VerificationResultNoCorruption
 	}
 	getCurrentExecutionRequest := &persistence.GetCurrentExecutionRequest{
+		ShardID:     shardID,
 		NamespaceID: executionInfo.NamespaceId,
 		WorkflowID:  executionInfo.WorkflowId,
 	}
@@ -721,11 +711,12 @@ func concreteExecutionStillExists(
 	executionInfo *persistencespb.WorkflowExecutionInfo,
 	executionState *persistencespb.WorkflowExecutionState,
 	shardID int32,
-	execStore persistence.ExecutionStore,
+	executionStore persistence.ExecutionStore,
 	limiter quotas.RateLimiter,
 	totalDBRequests *int64,
 ) (*ExecutionCheckFailure, bool) {
 	getConcreteExecution := &persistence.GetWorkflowExecutionRequest{
+		ShardID:     shardID,
 		NamespaceID: executionInfo.NamespaceId,
 		Execution: commonpb.WorkflowExecution{
 			WorkflowId: executionInfo.WorkflowId,
@@ -733,7 +724,7 @@ func concreteExecutionStillExists(
 		},
 	}
 	preconditionForDBCall(totalDBRequests, limiter)
-	_, err := execStore.GetWorkflowExecution(getConcreteExecution)
+	_, err := executionStore.GetWorkflowExecution(getConcreteExecution)
 	if err == nil {
 		return nil, true
 	}
@@ -762,6 +753,7 @@ func concreteExecutionStillOpen(
 	totalDBRequests *int64,
 ) (*ExecutionCheckFailure, bool) {
 	getConcreteExecution := &persistence.GetWorkflowExecutionRequest{
+		ShardID:     shardID,
 		NamespaceID: executionInfo.NamespaceId,
 		Execution: commonpb.WorkflowExecution{
 			WorkflowId: executionInfo.WorkflowId,
